
---
title: "resolver_map.proto"
weight: 5
---

<!-- Code generated by solo-kit. DO NOT EDIT. -->


### Package: `sqoop.solo.io` 
#### Types:


- [ResolverMap](#resolvermap) **Top-Level Resource**
- [TypeResolver](#typeresolver)
- [FieldResolver](#fieldresolver)
- [GlooResolver](#glooresolver)
- [RequestTemplate](#requesttemplate)
- [ResponseTemplate](#responsetemplate)
- [TemplateResolver](#templateresolver)
- [NodeJSResolver](#nodejsresolver)
  



##### Source File: [github.com/solo-io/sqoop/api/v1/resolver_map.proto](https://github.com/solo-io/sqoop/blob/master/api/v1/resolver_map.proto)





---
### ResolverMap

 
The ResolverMap object maps Resolvers to the fields in the GraphQL Schema
The skeleton of a Resolver Map will be generated by Sqoop automatically when a schema
is read or updated if one does not alreay exist.

```yaml
"types": map<string, .sqoop.solo.io.TypeResolver>
"status": .core.solo.io.Status
"metadata": .core.solo.io.Metadata

```

| Field | Type | Description | Default |
| ----- | ---- | ----------- |----------- | 
| `types` | `map<string, .sqoop.solo.io.TypeResolver>` | Types is a map of Type Names (defined in the schema) to a TypeResolver, which contain resolvers for the specific fields of the type |  |
| `status` | [.core.solo.io.Status](../../../../solo-kit/api/v1/status.proto.sk#status) | Status indicates the validation status of this resource. Status is read-only by clients, and set by gloo during validation |  |
| `metadata` | [.core.solo.io.Metadata](../../../../solo-kit/api/v1/metadata.proto.sk#metadata) | Metadata contains the object metadata for this resource |  |




---
### TypeResolver

 
TypeResolver contains the individual resolvers for each field for a specific type

```yaml
"fields": map<string, .sqoop.solo.io.FieldResolver>

```

| Field | Type | Description | Default |
| ----- | ---- | ----------- |----------- | 
| `fields` | `map<string, .sqoop.solo.io.FieldResolver>` | This is a map of Field Names to the resolver that Sqoop should invoke when a query arrives for that field |  |




---
### FieldResolver

 
Resolvers define the actual logic Sqoop needs to know in order to resolve a specific field query

```yaml
"glooResolver": .sqoop.solo.io.GlooResolver
"templateResolver": .sqoop.solo.io.TemplateResolver
"nodejsResolver": .sqoop.solo.io.NodeJSResolver

```

| Field | Type | Description | Default |
| ----- | ---- | ----------- |----------- | 
| `glooResolver` | [.sqoop.solo.io.GlooResolver](../resolver_map.proto.sk#glooresolver) | a GlooResolver, which leverages Gloo to retrieve data from backend services and functions for the query |  |
| `templateResolver` | [.sqoop.solo.io.TemplateResolver](../resolver_map.proto.sk#templateresolver) | a TemplateResolver, which uses Go Templates to generate data for the query |  |
| `nodejsResolver` | [.sqoop.solo.io.NodeJSResolver](../resolver_map.proto.sk#nodejsresolver) | a NodeJSResolver, which calls NodeJS functions to return data for the query |  |




---
### GlooResolver

 
GlooResolvers are the "meat" of Sqoop. GlooResolvers tell Sqoop how to invoke a "Gloo Function"

```yaml
"requestTemplate": .sqoop.solo.io.RequestTemplate
"responseTemplate": .sqoop.solo.io.ResponseTemplate
"action": .gloo.solo.io.RouteAction

```

| Field | Type | Description | Default |
| ----- | ---- | ----------- |----------- | 
| `requestTemplate` | [.sqoop.solo.io.RequestTemplate](../resolver_map.proto.sk#requesttemplate) | the Request Template, if specified, will become the body of the HTTP request used to invoke a function through Gloo input parameters, if needed, should be specified in the request template. See Sqoop's [Resolver documentation] for more information on Request Templates. |  |
| `responseTemplate` | [.sqoop.solo.io.ResponseTemplate](../resolver_map.proto.sk#responsetemplate) | The response template, if specified, will transform the body of HTTP responses returned by Gloo functions. This field should be used if the object returned by the Gloo Function does not match the type specified in the GraphQL schema. It can also be used to modify or transform responses from their original state. See Sqoop's [Resolver documentation] for more information on Response Templates. |  |
| `action` | [.gloo.solo.io.RouteAction](../../../../gloo/projects/gloo/api/v1/proxy.proto.sk#routeaction) | the routing action to take when resolver is executed. usually this is a Route destination |  |




---
### RequestTemplate



```yaml
"verb": string
"path": string
"body": string
"headers": map<string, string>

```

| Field | Type | Description | Default |
| ----- | ---- | ----------- |----------- | 
| `verb` | `string` |  |  |
| `path` | `string` |  |  |
| `body` | `string` |  |  |
| `headers` | `map<string, string>` |  |  |




---
### ResponseTemplate



```yaml
"body": string
"headers": map<string, string>

```

| Field | Type | Description | Default |
| ----- | ---- | ----------- |----------- | 
| `body` | `string` |  |  |
| `headers` | `map<string, string>` |  |  |




---
### TemplateResolver

 
A Go-template which will return data for a Resolver without making a function call. Template Resolvers can make use
of Sqoop's builtin template functions as well as the data provided by the Params object to the resolver.
Read more about Templates and Resolvers in Sqoop\'s [Resolver documentation].

```yaml
"inlineTemplate": string

```

| Field | Type | Description | Default |
| ----- | ---- | ----------- |----------- | 
| `inlineTemplate` | `string` | the Go template as an inline string |  |




---
### NodeJSResolver

 
NOTE: currently unsupported

```yaml
"inlineCode": string

```

| Field | Type | Description | Default |
| ----- | ---- | ----------- |----------- | 
| `inlineCode` | `string` |  |  |





<!-- Start of HubSpot Embed Code -->
<script type="text/javascript" id="hs-script-loader" async defer src="//js.hs-scripts.com/5130874.js"></script>
<!-- End of HubSpot Embed Code -->
